
# q-expansion of newform 1600.3.h.i, downloaded from the LMFDB on 03 June 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField, ZZ
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K([c/ZZ(den) for c in num]) for num, den in basis_data]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 1600
weight = 3
poly_data = [16, 0, -7, 0, 1]

# The entries in the following list give a basis for the
# coefficient ring in terms of a root of the defining polynomial above.
# Each line consists of the coefficients of the numerator, and a denominator.
basis_data  = [[[1, 0, 0, 0], 1], [[0, 11, 0, -1], 4], [[0, -15, 0, 5], 4], [[-35, 0, 10, 0], 1]]

hecke_ring_character_values = [[1151, [-1, 0, 0, 0]], [901, [1, 0, 0, 0]], [577, [-1, 0, 0, 0]]]
aps_data = [[0, 0, 0, 0], [0, 1, 0, 0], [0, 0, 0, 0], [0, -2, 0, 0], [0, 0, 0, 1], [0, 0, -4, 0], [0, 0, -3, 0], [0, 0, 0, -1], [0, -2, 0, 0], [-48, 0, 0, 0], [0, 0, 0, 2], [0, 0, 2, 0], [33, 0, 0, 0], [0, -16, 0, 0], [0, 4, 0, 0], [0, 0, -6, 0], [0, 0, 0, -4], [-38, 0, 0, 0], [0, -15, 0, 0], [0, 0, 0, -4], [0, 0, 1, 0], [0, 0, 0, 2], [0, 3, 0, 0], [-87, 0, 0, 0], [0, 0, -22, 0], [-42, 0, 0, 0], [0, -20, 0, 0], [0, 11, 0, 0], [58, 0, 0, 0], [0, 0, -27, 0], [0, 54, 0, 0], [0, 0, 0, -8], [0, 0, 21, 0], [0, 0, 0, -1], [-192, 0, 0, 0], [0, 0, 0, 2], [0, 0, -22, 0], [0, -33, 0, 0], [0, 76, 0, 0], [0, 0, 48, 0], [0, 0, 0, 11], [-22, 0, 0, 0], [0, 0, 0, 6], [0, 0, 59, 0], [0, 0, -66, 0], [0, 0, 0, 12], [0, 0, 0, 3], [0, 4, 0, 0], [0, -8, 0, 0], [-68, 0, 0, 0], [0, 0, -6, 0], [0, 0, 0, 8], [253, 0, 0, 0], [0, 0, 0, 19], [0, 0, 54, 0], [0, -26, 0, 0], [-312, 0, 0, 0], [0, 0, 0, 10], [0, 0, 88, 0], [198, 0, 0, 0], [0, 55, 0, 0], [0, 0, -78, 0], [0, -97, 0, 0], [0, 0, 0, 14], [0, 0, -10, 0], [0, 0, -60, 0], [0, 0, 0, -11], [0, 0, -25, 0], [0, 123, 0, 0], [242, 0, 0, 0], [0, 0, -30, 0], [0, 0, 0, -22], [0, 132, 0, 0], [0, 0, -50, 0], [0, 0, 0, -15], [0, -154, 0, 0], [18, 0, 0, 0], [0, 0, 20, 0], [-183, 0, 0, 0], [47, 0, 0, 0], [0, 0, 0, -21], [452, 0, 0, 0], [0, 0, 0, -2], [0, 0, 11, 0], [0, 0, 0, -32], [0, 83, 0, 0], [627, 0, 0, 0], [0, 0, 65, 0], [-12, 0, 0, 0], [0, 28, 0, 0], [0, 88, 0, 0], [0, 0, 0, -6], [0, -114, 0, 0], [0, 0, 0, 0], [0, 0, 0, 16], [0, 12, 0, 0], [-618, 0, 0, 0], [-423, 0, 0, 0], [0, -31, 0, 0], [8, 0, 0, 0], [0, 103, 0, 0], [0, 0, 132, 0], [0, -140, 0, 0], [-453, 0, 0, 0], [0, 0, 0, -56], [0, 0, -65, 0], [0, -195, 0, 0], [0, 0, -3, 0], [0, 0, 0, -54], [-637, 0, 0, 0], [0, -40, 0, 0], [0, 0, -182, 0], [0, 0, 126, 0], [0, 0, 0, 8], [0, 0, 0, 18], [858, 0, 0, 0], [0, 168, 0, 0], [0, 220, 0, 0], [0, 0, -198, 0], [0, 0, 0, -15], [-968, 0, 0, 0], [0, 0, 158, 0], [0, 0, 0, 0], [0, 121, 0, 0], [0, 0, 0, 69], [252, 0, 0, 0], [-1028, 0, 0, 0], [0, 0, 0, 62], [0, 176, 0, 0], [0, 0, 140, 0], [0, 0, 0, 0], [0, 190, 0, 0], [0, 0, 0, -56], [0, 0, -56, 0], [537, 0, 0, 0], [703, 0, 0, 0], [0, 0, -264, 0], [0, -176, 0, 0], [0, 0, -54, 0], [-522, 0, 0, 0], [0, 0, 0, 8], [282, 0, 0, 0], [0, -180, 0, 0], [0, -155, 0, 0], [748, 0, 0, 0], [0, 0, 0, 8], [0, 0, 286, 0], [0, 0, -195, 0], [0, 0, 0, 23], [0, -432, 0, 0], [0, 0, 44, 0], [-1602, 0, 0, 0], [0, -407, 0, 0], [0, -28, 0, 0], [0, -88, 0, 0], [0, 0, 0, -44], [0, 0, 0, -46], [-1518, 0, 0, 0], [0, 0, 7, 0], [468, 0, 0, 0], [0, -396, 0, 0], [0, 0, -297, 0], [0, -104, 0, 0], [0, 0, 0, 25], [0, 0, -351, 0], [0, 110, 0, 0], [0, 0, 0, 22], [0, 0, -340, 0], [1583, 0, 0, 0], [0, 0, 24, 0], [0, 0, 0, 33], [-902, 0, 0, 0], [0, 0, 0, -98], [0, 0, -167, 0], [0, 0, 0, 72], [-687, 0, 0, 0], [0, 0, 0, 11], [-882, 0, 0, 0], [0, -504, 0, 0], [-322, 0, 0, 0], [0, -516, 0, 0], [0, 0, 0, 5], [0, 0, 242, 0], [0, 0, 147, 0], [0, 204, 0, 0], [-108, 0, 0, 0], [0, 0, -158, 0], [0, -99, 0, 0], [-733, 0, 0, 0], [0, 0, 0, 40], [0, 0, 205, 0], [0, 319, 0, 0], [0, 0, 0, -37], [-2178, 0, 0, 0], [0, -557, 0, 0], [0, 0, 246, 0], [-517, 0, 0, 0], [0, 0, 374, 0], [0, 0, -129, 0], [0, 102, 0, 0], [2142, 0, 0, 0], [0, 0, 0, 76], [0, 0, -440, 0], [-187, 0, 0, 0], [0, 0, 0, 88], [0, 0, 42, 0], [0, 0, 0, -98], [0, -224, 0, 0], [138, 0, 0, 0], [0, 0, 0, -12], [0, 0, -374, 0], [762, 0, 0, 0], [0, -24, 0, 0], [0, 407, 0, 0], [0, 0, 0, -26], [1133, 0, 0, 0], [0, 206, 0, 0], [-2343, 0, 0, 0], [0, -422, 0, 0], [0, 0, -282, 0], [662, 0, 0, 0], [0, 0, 0, -28], [-1317, 0, 0, 0], [0, -162, 0, 0], [0, 391, 0, 0], [2618, 0, 0, 0], [0, 0, -291, 0], [0, 0, 0, 88], [0, 132, 0, 0], [0, 0, 0, -33], [0, 0, -442, 0], [0, 0, 0, 33], [0, 0, 0, -134], [-777, 0, 0, 0], [0, 692, 0, 0], [0, -70, 0, 0], [1607, 0, 0, 0], [0, 0, -24, 0], [0, 0, 0, -3], [0, 0, 0, -116], [0, -591, 0, 0], [0, 0, 0, -33], [0, -422, 0, 0], [1292, 0, 0, 0], [0, 0, -231, 0], [0, 0, 0, 64], [0, -550, 0, 0], [0, 0, 0, 8], [0, 0, 0, -55], [0, 176, 0, 0], [0, 0, -392, 0], [-213, 0, 0, 0], [0, -182, 0, 0], [-1243, 0, 0, 0], [0, 0, -240, 0], [0, 0, 0, 97], [968, 0, 0, 0], [0, -363, 0, 0], [0, 0, 18, 0], [0, 0, 253, 0], [0, 734, 0, 0], [0, -495, 0, 0], [1672, 0, 0, 0], [0, 0, -236, 0], [0, 0, 213, 0], [0, 0, 0, -93], [1122, 0, 0, 0], [1302, 0, 0, 0], [0, -783, 0, 0], [0, 0, 294, 0], [3028, 0, 0, 0], [0, 301, 0, 0], [0, 0, -371, 0], [0, 0, 0, 96], [0, 0, 458, 0], [0, 500, 0, 0], [0, -393, 0, 0], [1232, 0, 0, 0], [998, 0, 0, 0], [0, 0, 0, 125], [0, -268, 0, 0], [0, 0, 0, -66], [0, -464, 0, 0], [-332, 0, 0, 0], [0, -159, 0, 0], [0, 0, 0, -80], [0, 0, 154, 0], [0, 0, -72, 0], [0, 0, 0, -104], [-2178, 0, 0, 0], [-438, 0, 0, 0], [0, 24, 0, 0], [0, 0, 33, 0], [0, 0, 0, -77], [0, 0, 556, 0], [-2742, 0, 0, 0], [0, 0, 0, -62], [0, 0, -528, 0], [0, 0, 0, -48], [0, 317, 0, 0], [0, 0, 719, 0], [0, 0, 318, 0], [0, 0, 0, 184]]
